<?php
/**
 * General functions
 *
 * Contains globally available tool functions. It is included in every
 * page and sets up some defaults like error reporting, environment
 * setups and config loading
 *
 * @todo    Encapsulate httpClient and Cache as separate classes
 *
 * @package Core
 * @author  Andreas Goetz   <cpuidle@gmx.de>
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @version $Id: functions.php,v 1.42 2005/03/25 13:41:44 andig2 Exp $
 */

// add pwd to include_path
ini_set('include_path', '.:'.ini_get('include_path'));

/* This is needed to make sure we have our very own namespace without
   interfering Environment variables */
foreach (array_keys($_ENV) as $key)
{
    unset($GLOBALS[$key]);
}

require_once './config.inc.php';
require_once './core/constants.php';
require_once './core/template.php';
require_once './core/compatibility.php';
require_once './smarty/Smarty.class.php';

/* --------------------------------------------------------------------*/
// Set up some defaults

if ($config['debug']) {
	error_reporting(E_ALL ^ E_NOTICE);
} else {
	error_reporting(E_ERROR + E_PARSE);
}

if (get_magic_quotes_gpc())
{
	if (!empty($_REQUEST)) remove_magic_quotes($_REQUEST);
	ini_set('magic_quotes_gpc', 0);
}

set_magic_quotes_runtime(0);

// register_globals off? Well i like it...
extract($_REQUEST);

// security check
if (!empty($id))
{
    validate_input($id);
}    

// create smarty object
$smarty = new Smarty;
$smarty->compile_dir	= './cache/smarty';	            // path to compiled templates
$smarty->cache_dir		= './cache/smarty';	            // path to cached html
$smarty->plugins_dir    =  array('custom', 'plugins');  // add custom plugins folder
$smarty->use_sub_dirs	= 0;                            // restrict caching to one folder
$smarty->load_filter('output', 'trimwhitespace');       // remove whitespace from output

// load config
loadconfig();

// check authentification data for multiuser
if (basename($_SERVER['PHP_SELF']) != 'login.php') authcheck();



/* --------------------------------------------------------------------*/
// functions follow


/**
 * Used to remove magic quotes from the $_GET, $_POST, $_COOKIE and
 * $_SESSION super global arrays. It's automatically called in
 * functions.php
 *
 * @param array &$array Reference to an array
 */
function remove_magic_quotes(&$array)
{
	foreach (array_keys($array) as $key)
	{
		if (is_array($array[$key])) {
			remove_magic_quotes($array[$key]);
		} else {
			$array[$key] = stripslashes($array[$key]);
		}
	}
}

/**
 * magicquote safe formoutputter
 *
 * Strips slashes when magic_quotes_gpc is set.
 *
 * @param  string $name     The input string
 * @return string           The cleaned string
 */
function formvar($name)
{
	if (get_magic_quotes_gpc()) {
		$name = stripslashes($name);
	}
	return htmlspecialchars($name);
	#  return $name;
}

/**
 * Get high resolution time
 *
 * @return integer  current time in microseconds
 */
function getmicrotime()
{
	list($usec, $sec) = explode(' ',microtime());
	return ((float)$usec + (float)$sec);
}

/**
 * SQL function
 *
 * Wrapper for all Database accesses
 *
 * @param  string $sql_string The SQL-Statement to execute
 * @return mixed  either the resultset as an array with hashes or the insertid
 */
function runSQL($sql_string, $verify = true)
{
	global $config;
	
	if ($config['debug'])
    {
		echo "\n\n<!-- $sql_string -->\n\n";
		$timestamp = getmicrotime();
	}

	$link =	mysql_pconnect($config['db_server'], $config['db_user'], $config['db_password']) or
			errorpage("DB Connection Error",
					  "<p>Edit the database settings in <code>config.inc.php</code>.</p>
					   <p>Alternatively, consider running the <a href='install.php'>installation script</a>.</p>");

	$res  = mysql_db_query($config['db_database'],$sql_string,$link);

	// mysql_db_query returns either positive result ressource or true/false for an insert/update statement
	if ($res === false)
    {
		if ($verify)
        {
        	// report DB Problem
            errorpage('Database Problem', mysql_error($link)."\n<br />\n".$sql_string);
        }
        else 
        {
        	// ignore problem but forward the information 
        	$result = false;	
        } 
	}
	elseif ($res === true) 
	{
        // on insert, return id of created record
		$result = mysql_insert_id($link);
	} 
	else 
	{
        // return associative result array
        $result = array();
        
		for ($i=0; $i<mysql_num_rows($res); $i++) 
		{
            $result[] = mysql_fetch_assoc($res);
		}
		mysql_free_result($res);
	}
	
	if ($config['debug']) 
    {
		$timestamp = getmicrotime() - $timestamp;
		echo "\n\n<!-- time: $timestamp -->\n\n";
	}

	mysql_close($link);
	return $result;
}

/**
 * checks if the page is accessed from within the local net. 
 * If not, displays a simple error page and exits
 */
function localnet_or_die()
{
	if (!localnet()) errorpage('Forbidden', 'You are not allowed to access this page');
}

/**
 * Checks if the page is accessed from within the local net.
 *
 * @return  bool  true if localnet
 */
function localnet()
{
	global $config;
	return (preg_match('/'.$config['localnet'].'/', $_SERVER['REMOTE_ADDR']));
}

/**
 * Checks individual cache folder for existance
 * Checks if folder is writable
 * Creates folder if it doesn't exist 
 */
function verify_cache_folder($dir)
{
    if (!is_dir($dir)) 
    {
        if (!@mkdir($dir, 0700)) $error = 'Directory <code>'.$dir.'</code> does not exist.<br/>';
    } 
    elseif (!is_writable($dir)) 
    {
        $error = 'Directory <code>'.$dir.'</code> is not writable.<br/>';
    } 
    return $error;
}

/**
 * Checks if the cache directories exist and are writable by the webserver. 
 * If they don't exist it tries to create them. If this fails, too a simple
 * error page is displayed.
 * The function checks if the MySQL PHP extensions is loaded, too.
 */
function verify_installation()
{
    global $config;
    
    // check MySQL extension
    if (!extension_loaded('mysql'))
    {
        errorpage('MySQL extension for PHP not loaded',
                  '<p>The MySQL extension for PHP is not loaded.</p>
                   <p>Please make sure the MySQL module for PHP is installed and enabled
                   in your <code>php.ini</code></p>');
    }               

    // collect all directory-specific errors
    $error = '';
    
    // check cache
    foreach (array('cache' => 0,'cache/smarty' => 0, 'cache/imdb' => 1, 'cache/img' => 1) as $dir => $hierarchical)
	{
        // check top-level folders
        $error .= verify_cache_folder($dir);

        // check hierarchical folders
        if (empty($error) && $config['hierarchical'] && $hierarchical)
        {
            for ($i=0; $i<16; $i++)
            {
                $error .= verify_cache_folder($dir.'/'.dechex($i));
            }    
        }
    }
    
	if (!empty($error)) 
	{
        errorpage('Cache directories not writable',
                  '<p>The cache directories have to be writable by the webserver!</p>
                   <p>Please fix the following errors:</p>
                   <p>'.$error.'</p>');
    }
}

/**
 * Verify existance of cached file for given url/ extension
 *
 * @author Andreas Goetz <cpuidle@gmx.de>
 * @param  string	url of the image
 * @param  string	ext file extension of the image
 * @param  string	file result: URL to the cached image if exists
 * @return bool		result of check
 */
function checkCacheFile($url, $ext, &$file, $folder = 'img')
{
	global $config;

	// construct cache file name
	$md5    = md5($url);
	$subdir = ($config['hierarchical']) ? substr($md5, 0, 1).'/' : '';
	if ($ext) $ext = ".$ext";
	$file   = 'cache/'.$folder.'/'.$subdir.$md5.$ext;

	// return if existing
	$result = (file_exists($file) && filesize($file)) ? true : false;
	return($result);
}

/**
 * get Thumbnail-URL for an actor
 *
 * @param  string  name of the Actor
 * @return string  the URL to the cached image if exists or a link to img.php
 */
function getActorThumbnail($name, $actorid = 0)
{
	global $config;

	$SELECT = 'SELECT name, imgurl, UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(checked) AS cacheage
				 FROM '.TBL_ACTORS;

	// identify actor by unique actor id, of by name
	if ($actorid) {
		$result = runSQL($SELECT." WHERE actorid='".addslashes($actorid)."'");
	}
	if (!$actorid || empty($result[0]['name'])) {
		$result = runSQL($SELECT." WHERE name='".addslashes(html_entity_decode($name))."'");
	}

	$imgurl = 'img.php?name='.urlencode($name);
	if ($actorid) $imgurl .= '&actorid='.urlencode($actorid);

	if (!empty($result[0]['name']))
	{
		// really an image?
		if (preg_match('/\.(jpe?g|gif|png)$/i', $result[0]['imgurl'], $matches)) 
        {
			if (checkCacheFile($result[0]['imgurl'], $matches[1], $file)) return($file);
		} 
        elseif ($result[0]['cacheage'] <= $config['thumbAge']) 
        {
			// checked only recently
			return(img('nocover.gif'));
		}
	}
	return($imgurl);
}

/**
 * get Thumbnail for a movie
 *
 * @param  string   URL
 * @return string   the URL to the cached image if exists or a link to img.php
 */
function getThumbnail($imgurl)
{
    // really an image?
    if (preg_match('/\.(jpe?g|gif|png)$/i', $imgurl, $matches))
#    if (1==1)
	{
		// local file? - keep it!
		if (!preg_match('/^http/i', $imgurl)) return($imgurl);
        
        // file in cache?
		if (checkCacheFile($imgurl, $matches[1], $file))
        {
			return($file);
		} 
        else 
        {
            return('img.php?url='.urlencode($imgurl));
		}
	}
    else 
    {
		// no image url given -> nopic
		return(img('nocover.gif'));
	}
}

/**
 * Tries to find the given image in template directory then in the default
 * image directory.
 *
 * @param  string  filename of image
 * @return string  path to the image
 */
function img($img)
{
	global $config;

	$result = 'images/'.$img;
	if (file_exists($config['templatedir'].$result)) $result = $config['templatedir'].$result;
	return ($result);
}

/**
 * Load config options from config.inc.php and database and 
 * setup sane defaults.
 * Return configuration in global $config array variable
 *
 * @todo    Add security check if install.php is still available
 * @param   boolean force reload of configuration data
 */
function loadconfig($force_reload = false)
{
	global $config;
	global $lang;
	global $smarty;
/*
	// configuration cached and not outdated?
    if (isset($_SESSION['vdb']['config']) && (!$force_reload) && (!$config['recompile']) &&
       ($_SESSION['vdb']['config_timestamp'] == filemtime('./config.inc.php')))
	{
		// load from cache
        $config = $_SESSION['vdb']['config'];
	}
	else 
*/    
	{
        // check MySQL extension and cache directories
        verify_installation();
    
		// remember modification time
        $_SESSION['vdb']['config_timestamp'] = filemtime('./config.inc.php');

		// get config options from the database
		$SELECT = 'SELECT opt,value
                     FROM '.TBL_CONFIG;
		$result = runSQL($SELECT);
        
		// put db results into global config array  
		foreach ($result as $option) {
			$config[$option['opt']] = $option['value'];
		}
		
        // check if database matches the current version
        if ($config['dbversion'] < $config['dbneeded'])
        {
            // run installer
            header('Location: install.php?action=upgrade');
            exit;
/*        
            errorpage("Outdated Tablestructure", 
                      "<p>Your database does not match the current version of VideoDB! Please use the
                       <code>updatedb.sql</code> script to update your database!</p>
                       <p>On Unix you may use a line like that:<br /><br />
                       <code>mysql -p -f VideoDB < doc/updatedb.sql</code></p>
                       <p>Alternatively, consider running the <a href='install.php?action=upgrade'>installation script</a> 
                       to update your database structure.</p>");
*/
        }
        
		// get user config options from the database
		$user_id = $_COOKIE['VDBuserid'];
		if (!empty($user_id))
		{
			$SELECT = 'SELECT opt, value
                         FROM '.TBL_USERCONFIG.'
			            WHERE user_id = '.$user_id;

			$result = runSQL($SELECT);
			if (count($result))
            {
			    foreach ($result as $option) {
			        $config[$option[opt]] = $option[value];
			    }
			}
		}
		
		// set some defaults
		if (empty($config['language'])) $config['language'] = 'en';
		if (empty($config['template'])) $config['template'] = 'modern::compact';
		if (empty($config['filterdefault'])) $config['filterdefault'] = 'unseen';
        
//		if ($config['IMDBage'] < 1) $config['IMDBage']          = 60*60*24*5;
		if ($config['castcolumns'] < 1) $config['castcolumns']  = 4;
		if ($config['listcolumns'] < 1) $config['listcolumns']  = 1;
		if ($config['thumbAge'] < 1) $config['thumbAge']        = 60*60*24*7*3;
		if ($config['shownew'] < 1) $config['shownew']          = 12;
		
		// prepare som options for later use
		$config['languages']    = @split('::', $config['languageflags']);
		
		// prepare template/style
		$tpl                    = split('::', $config['template']);
		$config['style']        = 'templates/'.$tpl[0].'/'.$tpl[1].'.css';
		$config['templatedir']  = 'templates/'.$tpl[0].'/';
        
        // smarty cacheid for multiuser mode
        $config['cacheid']      = $tpl[0];

		// check if selected template is valid
		if (!file_exists($config['style']))
		{
			$config['style']       = 'templates/modern/compact.css';
			$config['template']    = 'modern::compact';
			$config['templatedir'] = 'templates/modern/';
		}

        // get installed engines meta information
        if (empty($config['engines']))
        {
            require_once './engines/engines.php';
            $config['engines'] = engineMeta();
        }
        
        // translate config options of type enginexyz into config[engine]
        foreach ($config['engines'] as $engine => $meta)
        {
            if ($config['engine'.$engine])
            {
                $config['engine'][$engine] = $config['engine'.$engine];
            }
        }    
/*
        // added proxy support for $_ENV
        $proxy = $config['proxy_host'];
        if (empty($proxy))
        {
            $env = array_change_key_case($_ENV);
            $proxy = $env['http_proxy'];
        }
        if (!empty($proxy))
        {
            $uri = parse_url($proxy);
            $config['proxy_host'] = ($uri['scheme']) ? $uri['host'] : $uri['path'];
            $config['proxy_port'] = ($uri['port']) ? $uri['port'] : 8080;
        }
*/

		// store loaded configuration           
        $_SESSION['vdb']['config'] = $config;
	}
	
	// setup smarty
	$smarty->template_dir = array($config['templatedir'], 'templates/modern');
	
	if ($config['recompile'])
    {
		$smarty->force_compile = 1;
	}
	
    // initialize languages
    $lang = array();

	// load english language as default
	require './language/en.php';

    // override it with local language if nessesary:
    if ($config['language'] != 'en') 
    {
        $languages = split('_', $config['language']);
        $file = '';
        foreach ($languages as $language)
        {
            if ($file) $file .= '_';
            $file .= $language;
            @include './language/'.$file.'.php';
        }
    }
}

/**
 * Downloads an URL to the given local file
 *
 * @param   string  $url    URL to download
 * @param   string  $local  Full path to save to
 * @return  bool            true on succes else false
 */
function download($url, $local)
{
	$resp = httpClient($url);
	if (!$resp['success']) return false;

	return(@file_put_contents($local, $resp['data']) !== false);
}

/**
 * Reads a saved HTTP response from a cachefile.
 * If caching is globally disabled ($config['IMDBage'] <= 0), file is not loaded.
 *
 * @param   string $url URL of the cached response
 * @return  mixed       HTTP Response, false on errors
 */
function getHTTPcache($url)
{
	global $config;

    $result = false;

    // only get file from cache if caching is enabled
    if ($config['IMDBage'] > 0)
    {
        if (checkCacheFile($url, null, $cfile, 'imdb') && (time()-filemtime($cfile) < $config['IMDBage'])) 
        {
            $resp = file_get_contents($cfile);
            if ($resp !== false) $resp = unserialize($resp);
            $result = $resp;
        }    
	}
    return $result;
}

/**
 * Saves a HTTP resonse to a cachefile
 * If caching is globally disabled ($config['IMDBage'] <= 0), file is not saved.
 *
 * @param  string $url  URL of the response
 * @param  mixed  $resp HTTP Response
 */
function saveHTTPcache($url, $resp)
{
    global $config;
    
    // only put file to cache if caching is enabled
    if ($config['IMDBage'] > 0)
    {
        // get the cache file name
        checkCacheFile($url, null, $cfile, 'imdb');

        // commit to disk
        file_put_contents($cfile, serialize($resp));
    }    
}


// decode content-encoding: gzip
function gzdecode(&$string)
{
	return gzinflate(substr($string, 10));
}

// convert cookies to http header
function cookies2header($cookies)
{
    global $request_cookies;
    
    // concatenate cookie string
	foreach ($cookies as $key => $val) 
    {
        // remember cookies for next request
        $request_cookies[$key] = $val;

        if ($headers) $headers .= '; ';
        $headers .= $key.'='.$val;
    }

    // build header
    if ($headers) $headers = 'Cookie: '.$headers."\r\n";
    return $headers;
}

/**
 * HTTP Client
 *
 * Returns the raw data from the given URL, uses proxy when configured
 * and follows redirects
 *
 * @author Andreas Goetz <cpuidle@gmx.de>
 * @param  string  $url      URL to fetch
 * @param  bool    $cache    use caching? defaults to false
 * @param  string  $post     POST data, if nonempty POST is used instead of GET
 * @param  integer $timeout  Timeout in seconds defaults to 15
 * @return mixed             HTTP response
 */
function httpClient($url, $cache=0, $post='', $timeout=15, $cookies = null, $headers2 = '')
{
    global $config;

    // since we shouldn't don't need session functionality here, 
    // use this as workaround for php bug #22526 session_start/popen hang 
    session_write_close();

    $method = 'GET';
    $headers = '';  // additional HTTP headers, used for post data
    if (!empty($post)) 
    {
        // POST request
        $method = 'POST';
        $headers .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $headers .= "Content-Length: ".strlen($post)."\r\n";
    }

    // get data from cache?
    if ($cache)
    {
        $resp = getHTTPcache($url.$post);
        if ($resp !== false) {
            return $resp;
        }
    }

    $response['error']  = '';
    $response['header'] = '';
    $response['data']   = '';
    $response['url']    = $url;
    $response['success']= false;

    $uri = parse_url($url);
    $server = $uri['host'];
    $path = $uri['path'];
    if (empty($path)) $path = '/'; #get root on empty path
    if (!empty($uri['query'])) $path .= '?'.$uri['query'];
    $port = $uri['port'];

    // proxy setup
    if (!empty($config['proxy_host']))
    {
        $request_url = $url;
        $server      = $config['proxy_host'];
        $port        = $config['proxy_port'];
        if (empty($port)) $port = 8080;
    }
    else
    {
        $request_url = $path;  // cpuidle@gmx.de: use $path instead of $url if HTTP/1.0
        $server      = $server;
        if (empty($port)) $port = 80;
    }

    // print "<pre>$request_url</pre>";

    // open socket
    $socket = fsockopen($server, $port);
    if (!$socket) 
    {
        $response['error'] = "Could not connect to $server";
        return $response;
    }
    socket_set_timeout($socket, $timeout);
    
    // build request
    $request  = "$method $request_url HTTP/1.0\r\n";
    $request .= "Host: ".$uri['host']."\r\n";
    $request .= "User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Windows 98)\r\n";
    if (extension_loaded('zlib')) $request .= "Accept-encoding: gzip\r\n";
    if ($cookies) $request .= cookies2header($cookies);
    $request .= "Connection: Close\r\n";
    $request .= $headers;
    $request .= $headers2;
    $request .= "\r\n";
    $request .= $post;

    // send request
    fputs($socket, $request);

    if ($config['debug']) echo "request:<br>".nl2br($request)."<p>";

    // log request
    if ($config['httpclientlog']) 
    {
        $log = fopen('httpClient.log', 'a');
        fwrite($log, $request."\n");
        fclose($log);
    }

    // read headers from socket
    do 
    {
        $response['header'] .= fread($socket,1);
    }
    while(!preg_match('/\r\n\r\n$/',$response['header']));

    // chunked encoding?
    if (preg_match('/transfer\-(en)?coding:\s+chunked\r\n/i',$response['header']))
    {
        do {
            unset($chunk_size);
            do 
            {
                $byte = fread($socket,1);
                $chunk_size .= $byte;
            } 
            while (preg_match('/[a-zA-Z0-9]/',$byte)); // read chunksize including \r
            
            $byte = fread($socket,1);     // readtrailing \n
            $chunk_size = hexdec($chunk_size);
            $this_chunk = fread($socket,$chunk_size);
            $response['data'] .= $this_chunk;
            if ($chunk_size) $byte = fread($socket,2); // read trailing \r\n
        } 
        while ($chunk_size);
    }
    else
    {
        // read entire socket
        while (!feof($socket)) 
        {
            $response['data'] .= fread($socket,4096);
        }
    }

    // close socket
    $status = socket_get_status($socket);
    fclose($socket);

    if ($config['debug']) echo "header:<br>".nl2br($response['header'])."<p>";
    // if ($config['debug']) echo "data:<br>".htmlspecialchars($response['data'])."<p>";

    // check for timeout
    if ($status['timed_out']) 
    {
        $response['error'] = "Connection timed out";
        return $response;
    }

    // log response
    if ($config['httpclientlog']) 
    {
        $log = fopen('httpClient.log', 'a');
        fwrite($log, $response['header']."\n");
        fclose($log);
    }

    // check server status code to follow redirect
    if (preg_match("/^HTTP\/1.\d 30[12].*?\n/s", $response['header']))
    {
        preg_match("/Location:\s+(.*?)\n/s",$response['header'],$matches);
        if (empty($matches[1])) 
        {
            $response['error'] = 'Redirect but no Location header found';
            return $response;
        } 
        else 
        {
            $location = trim($matches[1]);
            if (!preg_match("/^http/", $location)) $location = "http://".$uri['host'].$location;
            
            // don't use old headers again
            $headers	= '';
            
            // if this is a redirect, we must GET, not POST
            $post		= '';
            
            if (preg_match_all('/Set-Cookie:\s*(.+?);/', $response['header'], $_cookies, PREG_PATTERN_ORDER)) 
            {
                foreach ($_cookies[1] as $cookie) 
                {
                    // limit split to 2 elements (key/value)
                    list($key, $value) = split('=', $cookie, 2);
                    $cookies[$key] = $value;
                }
            }
            
            // perform redirected request
            $response = httpClient($location, $cache, $post, $timeout, $cookies);
            
            // remember we were redirected
            $response['redirect'] = $location;
            
            if ($response['success'] == true && $cache) saveHTTPcache($url.$post, $response);
            
            return $response;
        }
    }

    // verify status code
    if (!preg_match("/^.*? 200 .*?\n/s",$response['header'])) 
    {
        $response['error'] = 'Server returned wrong status.';
        return $response;
    }

    $response['success'] = true;
    
    // decode data if necessary- do not modify original headers
    if (preg_match("/Content-Encoding:\s+gzip\r?\n/i", $response['header'])) 
    {
        $response['data'] = gzdecode($response['data']);
    }

    // commit succesful request to cache
    if ($cache) saveHTTPcache($url.$post, $response);
    
    return $response;
}

/**
 * Displays an errorpage and exits
 *
 * @param string $title   The pages headline
 * @param string $body    An additional message
 */
function errorpage($title = 'An error occured', $body = '')
{   
    echo '<?xml version="1.0" encoding="en"?>';
?>
    <html xmlns="http:// www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
    <head>
        <title>VideoDB - ERROR</title>
        <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
        <meta name="description" content="VideoDB" />
    </head>
    <body>
        <h1><?=$title?></h1>
        <?=$body?>
    </body>
    </html>
<?
	exit;
}

/**
 * Checks if the logged in user has write permissions for the given video
 *
 * @param integer $id VideoID
 * @return boolean
 */
function check_videopermission($id)
{
	if (check_permission(AUTH_WRITEALL)) return true;
	if (!check_permission(AUTH_WRITE)) return false;
	
    $result = runSQL('SELECT '.TBL_USERS.'.name AS owner 
                        FROM '.TBL_DATA.', '.TBL_USERS.' 
                       WHERE '.TBL_DATA.'.id = '.$id.' AND '.TBL_USERS.'.id = '.TBL_DATA.'.owner_id');
	return ($result[0]['owner'] == $_COOKIE['VDBusername']);
}

/**
 * Used to check permissions on a user for a page
 *
 * @author Mike Clark <Mike.Clark@Cinven.com>
 * @param  integer $permission Permission to check
 * @return boolean             True if permission exists else false
 */
function check_permission($permission)
{
	global $config;
	
	if (!$config['multiuser']) return true;

	if (empty($_COOKIE['VDBuserid'])) {
		// not logged in
		$permissions = 0;
	} elseif (isset($_SESSION['vdb']['permissions'])) {
		// permission cache
		$permissions = $_SESSION['vdb']['permissions'];
	} else {
		$userid = $_COOKIE['VDBuserid'];
        $result = runSQL("SELECT permissions FROM ".TBL_USERS." WHERE id='$userid'");
		$permissions = $result[0]['permissions'];
		$_SESSION['vdb']['permissions'] = $permissions;
	}

	// check permissionbits
	return ($permissions & $permission);
}

/**
 * Check permissions on a user for a page and display error message on failure
 *
 * @param  integer $permission  Permission to check (admin,write,writeall)
 */
function permission_or_die($permission)
{
	if (!check_permission($permission))
	{
		errorpage("Access denied",'You don\'t have enough permissions to access this
				   page try to <a href="login.php">login</a> first.');
	}
}

/**
 * Checks if the user was authenticated and if the received auth cookie is valid. 
 * Function is called for every page except login.php!
 *
 * @param  string $redirect  Redirect to login page if authentication check unsuccessful
 */
function authcheck($redirect = true)
{
    global $config;
    
    // auth result will always be success for single-user
    $result = true;
    
    // auth check only in multiuser mode
    if ($config['multiuser']) 
    {
        $referer = substr($_SERVER['PHP_SELF'], strrpos($_SERVER['PHP_SELF'],'/')+1) .'?'. $_SERVER['QUERY_STRING'];
        
        // already logged in?
        $user   = $_COOKIE['VDBusername'];
        $pass   = $_COOKIE['VDBpassword'];
        $userid = $_COOKIE['VDBuserid'];
        
        // auth cookies present?
        if (!empty($user) && !empty($pass) && !empty($userid)) 
        {
            // There are some auth cookies - we need to check if they are valid
            if (!(eregi('[a-z]', $user) && eregi('[0-9]', $pass) && eregi('[0-9]', $userid)))
            {
                $result = false;
            }
            else
            {
                // This is the crucial bit, lets just test the cookiecode with SQL again.
                $row = runSQL("SELECT cookiecode FROM ".TBL_USERS." WHERE name='".addslashes($user)."' AND id=$userid");
                if ($row[0]['cookiecode'] !== $pass)
                {
                    $result = false;
                }
            }    
        } 
        else 
        {
            // No auth yet
            if ($config['denyguest'])
            {
                // guests are not allowed here!
                $result = false;
            }
        }

        // HTTP basic authentication (for RSS feed)?
        if (!$result && isset($_SERVER['PHP_AUTH_USER']) && isset($_SERVER['PHP_AUTH_PW']))
        {
            $user   = $_SERVER['PHP_AUTH_USER'];
            $pass   = $_SERVER['PHP_AUTH_PW'];
        
            // check if basic auth headers are valid
            if (eregi('[a-z]', $user))
            {
                $row = runSQL("SELECT passwd FROM ".TBL_USERS." WHERE name='$user'");

                // auth successful if password matches
                $result = (md5($pass) == $row[0]['passwd']);
            }            
        }
        
        // goto login page if anything was fishy
        if ($redirect &! $result)
        {
            header('Location: login.php?refer='.$referer);
            exit();
        }
    }
    
    return $result;
}

/**
 * Function to get the owner from videodata table
 *
 * @author Mike Clark <Mike.Clark@Cinven.com>
 * @param  integer $id       videodata id
 * @param  boolean $diskid   is the given ID a disk ID instead of videoID?
 * @return string  Returns the owner of the given Video or Disk
*/
function get_owner($id, $diskid = false)
{
    $SELECT = "SELECT ".TBL_USERS.".name AS owner 
                 FROM ".TBL_DATA.", ".TBL_USERS." 
                WHERE ".TBL_USERS.".id = ".TBL_DATA.".owner_id AND ".TBL_DATA.".";

	if ($diskid) {
		$SELECT .= "diskid = '$id'";
	} else {
		$SELECT .= "id = '$id'";
	}

	$result = runSQL($SELECT);
	return $result[0]['owner'];
}

/**
 * Get list of adult genre ids
 *
 * @return  array   Array of adult genre ids
 */
function get_adult_genres()
{
    global $config;
    
    $adultgenres = array();
    foreach(split('::',$config['adultgenres']) as $ag)
    {
        if (empty($ag)) continue;
        $adultgenres[] = $ag;
    }

    return $adultgenres;   
}

/**
 * Check if a movie is only for adults
 *
 * @param   integer $id   video id
 * @return  boolean       Returns true if access is granted
 */
function adultcheck($id)
{
	global $config;
	
	if (check_permission(AUTH_ADULT) || empty($config['adultgenres'])) 
    {
		// no multiuser or adult genres set or we actually do have the
		// permissions - whatever let's watch some pr0n ;-)
		return true;
	}
	
    $adultgenres = 'genre_id='.join(' OR genre_id=', get_adult_genres());
	$select = 'SELECT video_id
			   FROM '.TBL_VIDEOGENRE.'
			  WHERE video_id = '.$id.'
				AND ('.$adultgenres.')';
	$result = runSQL($select);	

	return(empty($result[0]['video_id']));
}

/**
 * Checks if the given movie was already seen by the logged in user. If no
 * user is logged in the $seen value is returned
 *
 * Gets username from cookie
 *
 * @param  integer $id    video id
 * @param  boolean $seen  seen
 * @return boolean        True if seen
 */
function get_userseen($id, $seen=0)
{
	if (empty($_COOKIE['VDBusername'])) return $seen;
	$SELECT =  'SELECT video_id 
                  FROM '.TBL_USERSEEN.', '.TBL_USERS.' 
                 WHERE '.TBL_USERSEEN.'.video_id='.$id." AND 
                       ".TBL_USERSEEN.".user_id = ".TBL_USERS.".id AND 
                       ".TBL_USERS.".name ='".$_COOKIE['VDBusername']."'";
	$result = runSQL($SELECT);

	$result = (count($result) > 0) ? 1 : 0;
	return($result);
}

/**
 * Sets the status in userseen accordingly to the given seen value
 *
 * Gets username from cookie
 *
 * @param  integer $id    video id
 * @param  boolean $seen  seen
 */
function set_userseen($id,$seen)
{
	if (!empty($_COOKIE['VDBusername']))
	{
		// get user id
        $SELECT = "SELECT id 
                     FROM ".TBL_USERS."
				    WHERE name='".$_COOKIE['VDBusername']."'";
		$result = runSQL($SELECT);
		$user_id= $result[0]['id'];

		if ($seen) {
            $SEEN = "REPLACE INTO ".TBL_USERSEEN." SET user_id=".$user_id.", video_id='".$id."'";
		} else {
            $SEEN = "DELETE FROM ".TBL_USERSEEN." WHERE user_id=".$user_id." AND video_id='".$id."'";
		}
		runSQL($SEEN);    
	}

/*
	// future code when userseen contains more user-specific data
	
	// record already exists?
	$SELECT = "SELECT seen, user_id FROM video_user, users ".
			"WHERE video_user.video_id=".$id." AND video_user.user_id = user.id AND users.user='".$_COOKIE['VDBusername']."'";
	$result = runSQL($SELECT);

	if (empty($result[0]['id'])) {
		$SELECT = "SELECT id FROM users ".
				  "WHERE user='".$_COOKIE['VDBusername']."'";
		$result = runSQL($SELECT);

		$SQL	= "INSERT INTO video_user SET user_id='".$result[0]['id']."', id='".$id."', seen='".$seen."'";
	}
	else {
		$SQL	= "UPDATE video_user SET seen='".$seen."' ".
				  "WHERE user_id='".$result[0]['id']."', id='".$id."', ";
	}
	runSQL($SQL);    
*/
}

/**
 * Verify variable is valid according to validation function
 *
 * @author Andreas Goetz <cpuidle@gmx.de>
 * @param  midex    variable to validate (e.g. $id)
 * @param  string   validation function name (e.g. is_numeric)
 */
function validate_input(&$var, $validation_func = 'is_numeric')
{
    if (function_exists($validation_func))
    {
        if (!$validation_func($var))
        {
            errorpage('Forbidden', 'You are not allowed to access this page.');
        }
    }
}


function smarty_display($template)
{
    global $smarty, $config;
    
    $smarty->display($template, null, $config['cacheid']);
}

?>
