<?php
/**
 * PDF Export functions
 *
 * Allows exporting movies to PDF
 * Requires FPDF libaray (http://www.fpdf.org)
 *
 * @package Core
 * @link    http://www.fpdf.org
 * @author  Andreas Gtz    <cpuidle@gmx.de>
 * @version $Id: pdf.php,v 1.12 2005/02/11 09:53:31 andig2 Exp $
 */

define('FPDF', $config['pdf_module']);
define('FPDF_FONTPATH', FPDF.'/font/');

require_once FPDF.'/fpdf.php';
require_once './core/functions.php';
require_once './engines/engines.php';
require_once './core/VariableStream.class.php';

/**
 * Copied from FPDF tutorial 3
 * enhanced with memory image creation for gif->png conversion
 *
 * @link    http://www.fpdf.org/?go=script&id=45
 */
class PDF extends FPDF
{
    var $B;
    var $I;
    var $U;
    var $HREF;
    var $GDCount = 0;
    var $Scale   = 0;      // dont rescale images
    
    function PDF($orientation='P', $unit='mm', $format='A4')
    {
        //Call parent constructor
        $this->FPDF($orientation,$unit,$format);
        //Initialization
        $this->B=0;
        $this->I=0;
        $this->U=0;
        $this->HREF='';
    }

    function GDImage($file, $x, $y, $w=0, $h=0, $im, $link='', $type='png')
    {
        //Put the GD image $im
        ob_start();
        $func = 'image'.$type;  // image creation function according to type
        $func($im);
        $data = ob_get_contents();      
        ob_end_clean();

        //create file-unique variable name to not duplicate images for pdf
        if (empty($file)) 
            $file = $this->GDCount++;
        else 
            $file = preg_replace('/[\s\/\.]/', '_', $file);

        $name = 'pdf_image'.$file;
        $GLOBALS[$name] = $data;

        //call Image using in-memory PNG file
        parent::Image('var://'.$name, $x, $y, $w, $h, $type, $link);
    }


    function Image($file, $x, $y, $w=0, $h=0, $ext='', $link='')
    {
        $image_types = array(1 => 'gif', 2 => 'jpg', 3 => 'png', 6 => 'bmp');

        list($width, $height, $ext, $attr) = getimagesize($file);
        $ext = $image_types[$ext];

        // find image loading function
        $func = $ext;
        switch($ext)
        {
/*
            case 'gif': $im = imagecreatefromgif($file);    break;
            case 'png': $im = imagecreatefrompng($file);    break;
            case 'jpg': $im = imagecreatefromjpeg($file);   break;
            case 'bmp': $im = imagecreatefromwbmp($file);   break;
*/
            case 'jpg': $func = 'jpeg'; break;
            case 'bmp': $func = 'wbmp'; break;
        }
        $func = 'imagecreatefrom'.$func;
        
        // check if loading functions exists (especiall for gif support)
        if (function_exists($func))
        {
            $im = $func($file);
        }
        else
        {
            $im = imagecreatetruecolor(1,1);
        }    
        
        // scaling requested?
        if ($this->Scale)
        {
            $this->max_width  = round($this->Scale * 95);
            $this->max_height = round($this->Scale * 135);

            $thumb_x = $width;
            $thumb_y = $height;

            // nothing should be 0 now
            if ($thumb_x > $this->max_width) {
                $thumb_y = round($thumb_y * $this->max_width / $thumb_x);
                $thumb_x = $this->max_width;
            } 
            if ($thumb_y > $this->max_height) {
                $thumb_x = round($thumb_x * $this->max_height / $thumb_y);
                $thumb_y = $this->max_height;
            }
        }
        
        // scaling requied?
        if (($this->Scale) && (($thumb_x != $width) || ($thumb_y != $height)))
        {
            // create white truecolor image (in case original is transparent)
            $target = imagecreatetruecolor($thumb_x, $thumb_y);
            $white  = imagecolorallocate($target, 255, 255, 255); 
            imagefilledrectangle($target, 0, 0, $thumb_x, $thumb_y, $white);
            imagecopyresampled($target, $im, 0,0, 0,0, $thumb_x,$thumb_y, $width,$height);  
            $this->GDImage($file, $x, $y, $w, $h, $target, $link, 'jpeg');  // change to png if you receive acrobat errors
            imagedestroy($target);
        }
        elseif ($ext == 'gif') {
            $this->GDImage($file, $x, $y, $w, $h, $im, $link);
        }
        else {
            parent::Image($file, $x, $y, $w, $h, $ext, $link);
        }
        
        imagedestroy($im);

/*
        if ($ext == 'gif')
        {
            // draw GD image directly
            $im = imagecreatefromgif($file);
            $this->GDImage($file, $x, $y, $w, $h, $im, $link);
            imagedestroy($im);
        }
        else
        {
            parent::Image($file, $x, $y, $w, $h, $ext, $link);
        }
*/
    }
    
    function VerifyFont($font, $mode = '')
    {
        $default_fonts  = array('Arial', 'Courier', 'Helvetica', 'Times');

        if (!in_array($font, $default_fonts))
        {
            if ($mode)
                $this->AddFont($font, 'B', strtolower($font).'b.php');
            else
                $this->AddFont($font);            
        }
    }
    
    function WriteHTML($html)
    {
        //HTML parser
        $html=str_replace("\n",' ',$html);
        $a=preg_split('/<(.*)>/U',$html,-1,PREG_SPLIT_DELIM_CAPTURE);
        foreach($a as $i=>$e)
        {
            if($i%2==0)
            {
                //Text
                if($this->HREF)
                    $this->PutLink($this->HREF,$e);
                else
                    $this->Write(5,$e);
            }
            else
            {
                //Tag
                if($e{0}=='/')
                    $this->CloseTag(strtoupper(substr($e,1)));
                else
                {
                    //Extract attributes
                    $a2=explode(' ',$e);
                    $tag=strtoupper(array_shift($a2));
                    $attr=array();
                    foreach($a2 as $v)
                        if(ereg('^([^=]*)=["\']?([^"\']*)["\']?$',$v,$a3))
                            $attr[strtoupper($a3[1])]=$a3[2];
                    $this->OpenTag($tag,$attr);
                }
            }
        }
    }

    function OpenTag($tag,$attr)
    {
        //Opening tag
        if($tag=='B' or $tag=='I' or $tag=='U')
            $this->SetStyle($tag,true);
        if($tag=='A')
            $this->HREF=$attr['HREF'];
        if($tag=='BR')
            $this->Ln(5);
    }

    function CloseTag($tag)
    {
        //Closing tag
        if($tag=='B' or $tag=='I' or $tag=='U')
            $this->SetStyle($tag,false);
        if($tag=='A')
            $this->HREF='';
    }

    function SetStyle($tag,$enable)
    {
        //Modify style and select corresponding font
        $this->$tag+=($enable ? 1 : -1);
        $style='';
        foreach(array('B','I','U') as $s)
            if($this->$s>0)
                $style.=$s;
        $this->SetFont('',$style);
    }

    function PutLink($URL,$txt)
    {
        //Put a hyperlink
        $this->SetTextColor(0,0,255);
        $this->SetStyle('U',true);
        $this->Write(5,$txt,$URL);
        $this->SetStyle('U',false);
        $this->SetTextColor(0);
    }
}

/**
 * Export PDF document
 *
 * @param   string  $where  WHERE clause for SQL statement
 */
function pdfexport($WHERE)
{
    global $config;
    
    $ypos           = 10;
    $margin         = 5;
    $left_margin    = 5;
    $font_size      = 10;

    $image_height   = 24;
    $image_width    = (int) 95 / 135 * $image_height;

    $font_title     = $config['pdf_font_title'];
    $font_plot      = $config['pdf_font_plot'];
    
    $text_length    = 500;      

    // setup pdf class
    $pdf = new PDF();
    $pdf->VerifyFont($font_title);
    $pdf->VerifyFont($font_title, 'B');
    $pdf->VerifyFont($font_plot);
    $pdf->AddPage();
    // add downscaling
    if ($config['pdf_scale']) 
    {
        $pdf->Scale     = $config['pdf_scale'];
        $pdf->max_width = 95;
        $pdf->max_height= 135;
    }    

    // includes genres table in query?
    $TABLE = '';
    if (strstr($WHERE, 'genre_id'))
    {
        $TABLE  = TBL_GENRE.', ';
        $WHERE .= ' AND '.TBL_DATA.'.id = '.TBL_GENRE.'.video_id';
    }

    $SQL = 'SELECT '.TBL_DATA.'.*, 
                   '.TBL_USERS.'.name AS owner, 
                   '.TBL_MEDIATYPES.'.name AS mediatype
              FROM '.$TABLE.TBL_DATA.'
         LEFT JOIN '.TBL_USERSEEN.' ON '.TBL_DATA.'.id = '.TBL_USERSEEN.'.video_id AND 
                   '.TBL_USERSEEN.'.user_id = \''.addslashes($_COOKIE['VDBuserid']).'\'              
         LEFT JOIN '.TBL_USERS.' ON '.TBL_DATA.'.owner_id = '.TBL_USERS.'.id
         LEFT JOIN '.TBL_MEDIATYPES.' ON mediatype = '.TBL_MEDIATYPES.'.id
             WHERE '.$WHERE;

    $result = runSQL($SQL);

    foreach ($result as $row)
    {
        $title = $row['title'];
        if ($row['subtitle'])   $title .= ' - '.$row['subtitle'];
        if ($row['diskid'])     $title .= ' ['.$row['diskid'].']';

        // get drilldown url for image
        $imdb = $row['imdbID'];
        $link = ($imdb) ? engineGetContentUrl($imdb, engineGetEngine($imdb)) : '';
        
        // title
        $pdf->SetFont($font_title, 'B', $font_size);
        $pdf->SetXY($left_margin + $image_width + $margin, $ypos);
        $pdf->Cell(0, 0, html_entity_decode($title), 0,1, 'L',0,$link);
   
        // plot
        $pdf->SetFont($font_plot, '', $font_size-1);
        $pdf->SetXY($left_margin + $image_width + $margin, $ypos+3);
        $pdf->SetLeftMargin($left_margin + $image_width + $margin);
        $plot = leftString(html_entity_decode($row['plot']), $text_length);
        $pdf->WriteHTML($plot);

        // image
        $url    = $row['imgurl'];
        $file   = 'images/nocover.gif';
        
        if (preg_match('/\.(jpe?g|gif|png)$/i', $url, $matches))
        {
            // image via http?
            if (preg_match('/^http/i', $url)) 
            {
                // cached locally?
                if (checkCacheFile($url, $matches[1], $local_file)) 
                {
                    $file = $local_file;
                }    
            }
            elseif (file_exists($url)) 
            {
                // local file
                $file = $url;
            }
        }
        
        // image file present?
        if ($file)
        {
            $pdf->Image($file, $left_margin, $ypos-2, $image_width, $image_height, '', $link);    
        }    

        // add mediatype image
        $mediatype = $row['mediatype'];
        if (preg_match("/^(DVD|DivX|CD|VCD|SVCD|VHS)/i", $mediatype, $matches)) 
        {
            $type_image = strtolower($matches[1]).'.png';
        }
        else $type_image = '';

        if ($type_image)
        {
            $pdf->Image('./images/media/'.$type_image, 190, $ypos-2, 8, 0, '', '');
        }

        // new position
        $ypos += $margin;
        if ($file or $plot) 
        {
            $ypos += $image_height;
        }
        else
        {
            $ypos += $font_size;
        }

        if ($ypos > 250) 
        {   
            $ypos = 10;
            $pdf->AddPage();
        }    
    }

    $pdf->Output('videoDB.pdf', 'D');
}

?>