<?php
/**
 * Edit Page
 *
 * The edit form for adding and editing video data
 *
 * @todo    Check if 0000 instead of empty year is needed
 *
 * @package videoDB
 * @author  Andreas Gohr <a.gohr@web.de>
 * @version $Id: edit.php,v 2.54 2005/03/25 12:16:08 andig2 Exp $
 */

require_once './core/functions.php';
require_once './core/genres.php';
require_once './core/custom.php';
require_once './core/security.php';

// change this if you have some fancy naming style
$NEXTUSERID = "SELECT lpad(max(diskid)+1, 4, '0') AS max FROM ".TBL_DATA;
/*
    this scheme was suggested by Elkin Fricke see
    https://sourceforge.net/mailarchive/forum.php?thread_id=3404620&forum_id=34922

    $NEXTUSERID = "SELECT concat(lpad(max(substring(diskid,1,5)+1),5,'0'),'-01' ) AS max FROM ".TBL_DATA;
*/

$articles = array('the ', 'la ', 'a ', 'der ', 'die ', 'das ', 'des ', 'dem ', 'den ', 
                  'ein ', 'eine ', 'eines ', 'le ', 'el ', "l'", 'il ', 'les ', 'i ', 
                  'o ', 'un ', 'los ', 'de ', 'an ', 'una ', 'las ', 'gli ', 'het ', 
                  'lo ', 'os ', 'az ', 'ha-', 'een ', 'det ', 'oi ', 'ang ', 'ta ', 
                  'al-', 'uno ', "un'", 'ett ', 'mga ', ' ', ' ', 'els ', ' ', ' ');

            
// check for localnet
localnet_or_die(); 

// multiuser permission check
$required_permisson = (empty($id) || (get_owner($id) == $_COOKIE['VDBusername'])) ? AUTH_WRITE : AUTH_WRITEALL;
permission_or_die($required_permisson);

// XML import
if ($config['xml'] && ($import == 'xml'))
{
    require_once './core/xml.php';
    
    // xml file upload
    if (isset($_FILES['xmlfile']) && is_uploaded_file($_FILES['xmlfile']['tmp_name']))
    {
        $file    = $_FILES['xmlfile']['tmp_name'];
        $xmldata = file_get_contents($file);
        unlink($file);
    }

    // uploading XML data directly or loaded from file
    if (!empty($xmldata))
    {
        $error      = '';
        $item_id    = 0;
        
        if (($xmlitems = xmlimport($xmldata, $error)) !== false)
        {
            // multiple items imported
            if ($xmlitems === true)
            {
                header('Location: index.php?filter=new');
                exit;
            }
            // exactly one movie imported?
            else
            {
                header('Location: show.php?id='.$xmlitems);
                exit;
            }
        }
        $smarty->assign('xmlerror', $error);
    }
    
    // prepare templates
    tpl_page();

    // display templates
    tpl_display('xmlimport.tpl');
    exit;
}

// list of fields to be read/written from/to html form
$imdb_set_fields    = array('md5','title','subtitle','language','diskid','mediatype','comment','disklabel',
                            'imdbID','year','imgurl','director','actors','runtime','country','plot','filename',
                            'filesize','filedate','audio_codec','video_codec','video_width','video_height','istv',
                            'custom1','custom2','custom3','custom4','seen');

// legacy
if ($imdb) $lookup = 1;

// get default lookup mode (0=ignore, 1=lookup, 2=overwrite) if not set
if (!isset($lookup)) $lookup = $config['lookupdefault'];

// lookup imdb
if ($lookup && $imdbID)
{
    require_once './engines/engines.php';

    // get engine from id
    if (empty($engine)) $engine = engineGetEngine($imdbID);

    // get external data
    $imdbdata = engineGetData($imdbID, $engine);

    // lookup cover
    if (empty($imgurl) || ($lookup > 1)) 
    {
        $imgurl = $imdbdata['coverurl'];
    }

    // lookup genres
    if (count($genres) == 0 || ($lookup > 1)) 
    {
        $genres = array();
        $gnames = $imdbdata['genres'];
        if (isset($gnames)) 
        {
            foreach ($gnames as $gname) 
            {
                $genres[] = getGenreId($gname);
            }
        }
    }

    // lookup actors
    if (empty($actors) || ($lookup > 1)) 
    {
        $actors = $imdbdata['cast'];
    }

    // lookup all other fields
    foreach (array_keys($imdbdata) as $name) 
    {
        if (in_array($name, array('coverurl', 'genres', 'cast'))) continue;

        // use !$$ as empty($$) doesn't seem to work
        if (!$$name || ($lookup > 1)) 
        {
            $$name = $imdbdata[$name];
        }
    }
    
    // custom fields
    for ($i=1; $i<=4; $i++)
    {
        $custom = 'custom'.$i;
        $type   = $config[$custom.'type'];
        if (!empty($type) && $$type)
        {
            // copy imdb data into corresponding custom field
            $$custom = $$type;
        }
    }   
}

// get fields from db if copying
if ($copy && $copyid)
{
    $video = runSQL('SELECT * FROM '.TBL_DATA.' WHERE id = '.$copyid);

    // get fields (according to list) from db to be saved later
    foreach ($video[0] as $name => $val) 
    {
        // don't copy diskid
        if ($name == 'diskid') continue;
        if (in_array($name, $imdb_set_fields)) $$name = $val;
    }
}

// save data
if ($save)
{
    // use 0000 for empty year- if not set, current year is used
    if (empty($year)) $year = '0000';

    // update all fields according to list
    foreach ($imdb_set_fields as $name)
    {
        // sanitize input
        $$name = removeEvilTags($$name);
        
        // make sure no formatting contained in basic data
        if (in_array($name, array('title', 'subtitle')))
        {
            $$name = trim(strip_tags($$name));
            
            // string leading articles?
            if ($config['removearticles'])
            {
                foreach ($articles as $article)
                {
                    if (preg_match("/^$article+/i", $$name))
                    {
                        $$name = trim(preg_replace("/(^$article)(.+)/i", "$2, $1", $$name));
                        break;
                    }
                }    
            }    
        }
        
        if ($SETS) $SETS .= ', ';
        $SETS .= "$name = '".addslashes($$name)."'";
    }

    // explicit setting of owner only for admins
    if (!check_permission(AUTH_ADMIN) || empty($owner_id))
    {
        $owner_id = $_COOKIE['VDBuserid'];
    }
    $SETS = "owner_id = '".addslashes($owner_id)."', ".$SETS;

    if ($id) 
    {
        // update existing record
        // if the disk was on the wishlist and is now available, make sure it appears under 'new'
        if (($oldmediatype == MEDIA_WISHLIST) && ($mediatype != MEDIA_WISHLIST)) {
            $SETS .= ', created = NOW()';
        }

        $INSERT = 'UPDATE '.TBL_DATA.' SET '.$SETS.' WHERE id = '.$id;
        runSQL($INSERT);
    } 
    else 
    {
        // insert new record
        $INSERT = 'INSERT INTO '.TBL_DATA.' SET '.$SETS.', created = NOW()';
        $id = runSQL($INSERT);
    }
    

    // save genres
    setItemGenres($id, $genres);

    // set seen for currently logged in user
    set_userseen($id, $seen);

    // uploaded cover?
    if (isset($_FILES['coverupload']) && is_uploaded_file($_FILES['coverupload']['tmp_name'])) 
    {
        $file = $_FILES['coverupload']['tmp_name'];
        $mime = $_FILES['coverupload']['type'];

        // determine file extension
        if (preg_match('=image/jpe?g=i', $mime)) {
            $ext = 'jpg';
        } elseif (preg_match('=image/gif=i', $mime)) {
            $ext = 'gif';
        } elseif (preg_match('=image/png=i', $mime)) {
            $ext = 'png';
        } elseif (preg_match('=application/octet-stream=i', $mime)) {
            if (preg_match("/\.(jpe?g|gif|png)$/i", $_FILES['coverupload']['name'], $matches)) {
                $ext = $matches[1];
            }
        }

        // move to cache and update db
        if (!empty($ext)) 
        {
            $coverfile = 'cache/img/'.$id.'.'.$ext;
            if (move_uploaded_file($file, $coverfile)) 
            {
                $sql = "UPDATE ".TBL_DATA." SET imgurl='$coverfile' WHERE id=$id";
                runSQL($sql);
            }
        }
    }

    // add another?
    if ($add_flag)
    {
        // remove id to prevent edit mode instead of new
        $id = '';
        $smarty->assign('add_flag', $add_flag); 
    }
    else
    {
        // show the saved movie
        header('Location: show.php?id='.$id);
        exit();
    }    
}


// load existing data
if ($id)
{
	// select all fields according to list, plus id
	foreach ($imdb_set_fields as $name)
    {
		if ($SELECT) $SELECT .= ', ';
		$SELECT .= $name;
	}
    
    $SELECT = 'SELECT '.TBL_DATA.'.id, '.TBL_DATA.'.owner_id, '.TBL_USERS.'.name AS owner, '.$SELECT.'
                 FROM '.TBL_DATA.'
                 LEFT JOIN '.TBL_USERS.' ON '.TBL_DATA.'.owner_id = '.TBL_USERS.'.id
                WHERE '.TBL_DATA.'.id = '.$id;
	$video = runSQL($SELECT);

	// get seen status for currently logged in user
	$video[0]['seen']	= get_userseen($id,$video[0]['seen']);
	
	// diskid to global scope:
	$diskid = $video[0]['diskid'];
} else {
	$video[0]['language'] = $config['langdefault'];
}

// assign automatic disk id
if (($config['autoid']) && (empty($diskid)))
{
	$result = runSQL($NEXTUSERID);
	$video[0]['diskid'] = $result[0]['max'];
}

/*print '<pre>';
print_r($video);
print '</pre>';*/

// prepare templates
tpl_page();
tpl_edit($video);

$smarty->assign('lookup_id', $lookup);
$smarty->assign('http_error', $CLIENTERROR);

// allow XML import
if ($config['xml'] && empty($id)) $smarty->assign('xmlimport', true);

// display templates
tpl_display('edit.tpl');

?>
