<?php
/**
 * Amazon Parser
 *
 * Parses data from Amazon
 *
 * @package Engines
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @link    http://www.amazon.de Amazon
 * @version $Id: amazon.php,v 1.5 2005/02/23 20:32:15 andig2 Exp $
 */

$GLOBALS['amazonServer']	= 'http://www.amazon.de';

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function amazonMeta()
{
    return array('name' => 'Amazon', 'stable' => 1);
}

/**
 * Get search Url for an Amazon product
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function amazonSearchUrl($title)
{
	global $amazonServer;
	return $amazonServer;
}

/**
 * Search a Movie/DVD/Book etc
 *
 * Searches for a given title on Amazon and returns the found links in
 * an array
 *
 * @author  Andreas Goetz (cpuidle@gmx.de)
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function amazonSearch($title)
{
	global $amazonServer, $cache;
	global $CLIENTERROR;

	$post = 'size=10'.
			'&url='.urlencode('index=blended').
			'&field-keywords='.urlencode($title);
	
	$resp = httpClient($amazonServer.'/exec/obidos/search-handle-form/ref=sr_sp_go_qs/028-5046340-6062930', $cache, $post);
	if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

	if (preg_match_all('/<a href=\/exec\/obidos\/ASIN\/(.+?)\/.*?>(.+?)<\/a>/is', $resp['data'], $data, PREG_SET_ORDER)) {
		foreach ($data as $row) {
			if (ereg('<img', $row[2])) continue;
			$info['id']			= trim($row[1]);
#			$info['showid']		= $row[2];
#			$info['episodeid']	= $row[3];
			$info['title']		= trim($row[2]);
#			$info['subtitle']	= $row[4];
			$ary[]				= $info;
		}
	}
#die(nl2br(print_r($resp,1)));

	return $ary;	
}

/**
 * Fetches the data for a given Amazon ID (equals ISBN)
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   int   IMDB-ID
 * @return  array Result data
 */
function amazonData($imdbID)
{
	global $amazonServer, $cache;
	global $CLIENTERROR;

	$data = array();  //result

	$languages = array('deutsch' => 'german', 'englisch' => 'english');
	$genres = array('Thrillers' => 'Thriller', 'Krimis' => 'Crime', 'Literature' => '', 'Science Fiction' => 'Sci-Fi', 'Children\'s Books' => 'Children',
			  'Reference' => 'Documentary', 'Naturwissenschaften' => 'Sciences', 'Technik' => '',
			  'Politik' => '', 'Biografien' => '', 'Geschichte' => '', 'Fachbcher' => '');

	//fetch mainpage
	$resp = httpClient($amazonServer.'/exec/obidos/ASIN/'.$imdbID, 1);
	if (!$resp[success]) {
		$CLIENTERROR .= $resp[error]."\n";
		return $data;
	}
	# print htmlspecialchars($resp[data]);

	//Cover-URL (verify if large image exists)
	if (preg_match("/\"(http:\/\/images-eu.amazon.com\/images\/P\/$imdbID.0\d.\wZZZZZZZ.jpg)\"/",$resp[data],$ary)) {
		#print_r($ary);
		$data[coverurl] = $ary[1];
	}

	//Director
	preg_match("/<a.*?search-handle-url.*?field-author.*?>(.*?)<\/a>/is",$resp[data],$ary);
	$data[director] = trim($ary[1]);

	// Language
	preg_match("/<b>Sprache:<\/b>(.*?)<br>/is",$resp[data],$ary);
	$data[language] = trim(strtolower($ary[1]));
	if ($languages[$data[language]]) $data[language] = $languages[$data[language]];

	//Year
	# <b>Erscheinungsdatum:</b> 1. Januar 1994<br>
	preg_match("/<b>Erscheinungsdatum:<\/b>.*?(\d\d\d\d)<br>/is",$resp[data],$ary);
	$data[year]     = trim($ary[1]);

	//Plot
	# <a name="05532311624500">
	# <span class="small"><b><i>Synopsis</i></b></span><br>
	# <span class="serif">When a woman claiming to be Nero Wolfe's long-lost daughter gets into trouble over some missing diamonds, Wolfe, along with his sidekick, Archie, becomes involved in a world of international intrigue. Reissue.</span>
	preg_match('/<b><i>Synopsis<\/i><\/b>.*?<span class="serif">(.*?)<\/span>/is',$resp[data],$ary);
	$data[plot] = trim($ary[1]);

	//Rating
	preg_match('/<b>Durchschnittliche Kundenbewertung:<\/b>.*?alt="(\d\.\d).*?"/is',$resp[data],$ary);
	$data[rating]   = 2 * trim($ary[1]);

	//Genres (as Array)
	preg_match('/<b>Kategorie\(n\):<\/b>.*?>(.*?)<br>/is',$resp[data],$ary);
	$gens = preg_split("/&|,/", $ary[1]);
	#print_r($gens);
	foreach($gens as $genre) {
		$genre = trim($genre);
		$genre = strip_tags($genre);
		if (!$genre) continue;
		if (isset($genres[$genre])) $genre = $genres[$genre];
		#print_r($genre);
		if (!$genre) continue;
		$data[genres][] = $genre;
	}
	#print_r($data[genres]);

	return $data;
}

?>