<?php
/**
 * DVD Palace Parser
 *
 * Parse data from a german DVD Database
 *
 * @package Engines
 * @link    http://www.dvd-palace.de
 */

$GLOBALS['dvdpalaceServer']	= 'http://www.dvd-palace.de';
$GLOBALS['resultsPerPage']	= 10; // DVD Palace Config

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function dvdpalaceMeta()
{
    return array('name' => 'DVD Palace (de)');
}

/**
 * Get Url to search DVD Palace for a movie
 * 
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function dvdpalaceSearchUrl($title)
{
	global $dvdpalaceServer;
	return $dvdpalaceServer.'/dvddatabase/dbsearch.php?action=1&suchbegriff='.urlencode($title);
}

/**
 * Get Url to visit DVD Palace for a specific movie
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string	$id	The movie's external id
 * @return  string		The visit URL
 */
function dvdpalaceContentUrl($id)
{
	global $dvdpalaceServer;
	$id = preg_replace('/^DP/', '', $id);
	return $dvdpalaceServer.'/dvddatabase/getinfo.php?dvdid='.$id;
}

/**
 * Search a Movie
 *
 * Searches for a given title on the DVD Palace and returns the found links in
 * an array
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function dvdpalaceSearch($title)
{
    global $dvdpalaceServer;
    global $resultsPerPage;
    global $CLIENTERROR;
    $pageNo = 1;
    while(1) {
		$url = $dvdpalaceServer.'/dvddatabase/dbsearch.php?action=1&suchbegriff='.urlencode($title).'&start='.(($pageNo - 1) * $resultsPerPage);
	    $resp = httpClient($url, 1);
	    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";
	    $resp['data'] = preg_replace('/[\r\n\t]/',' ', $resp['data']);
	
	    // direct match (redirecting to individual title)?
	    $single = array();
	    if (preg_match('/^'.preg_quote($dvdpalaceServer,'/').'\/dvddatabase\/getinfo.php\?dvdid=([0-9]+)/i', $resp['url'], $single))
	    {
			if(preg_match("/<TITLE>.*?\+\+\+(.*?)\+\+\+.*<\/TITLE>/i", $resp['data'], $matches)) {
		        $ary[0]['id']   = "DP".$single[1];
	        	$ary[0]['title']= trim($matches[1]);
			}
	        break;
	    }
	    
	    // multiple matches
		if (preg_match_all('/getinfo.php\?dvdid=([0-9]+)">(\w.*?)<.*?<TD.*?<TD.*?<b>(.*?)<\/b>.*?<TD.*?>&nbsp;(.*?)<.*?<td.*?<IMG.*?fsk(.*?)logo.*?<TD.*?>(.*?)<\/td>/i', $resp['data'], $data, PREG_SET_ORDER)) {
		    foreach ($data as $row) 
		    {
		    	if (strtoupper(trim($row[5])) == "OA" ) $fsk= "";
		    	else $fsk= "FSK".trim($row[5])." ";
		    	
		    	if (preg_match('/<b>(.*?)<\/b>/i',$row[6],$m)) {
		    		$min = " ".trim($m[1])."min";
		    	} 
		    	else $min = "";
		    	
		        $info['id']     = 'DP'.$row[1];
		        $info['title']  = trim($row[2])." (".trim($row[3]).") ".$fsk.trim($row[4]).$min;
		        
		        $ary[]          = $info;
		    }
		}
		
		if (preg_match('#Seiten \(<b>([0-9]+)</b>\)#i', $resp['data'], $single)) {
			if ($pageNo == $single[1]) break; 
			else $pageNo++;  
		}
		else break;
    }	
	return $ary;
	
}

/**
 * Fetches the data for a given DVD Palace-ID
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   int   DVD Palace-ID
 * @return  array Result data
 */
function dvdpalaceData($dvdpalaceID) 
{
    global $dvdpalaceServer;
    global $CLIENTERROR;
    $dvdpalaceID = preg_replace('/^DP/', '', $dvdpalaceID);

    $data= array();	// result
    $ary = array();	// temp

    // fetch mainpage
    $resp = httpClient($dvdpalaceServer.'/dvddatabase/getinfo.php?dvdid='.$dvdpalaceID, 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";
    $resp['data'] = preg_replace('/[\r\n\t]/',' ', $resp['data']);

    // Titles
    preg_match("/<TITLE>.*?\+\+\+(.*?)\+\+\+.*?<\/TITLE>/i", $resp['data'], $ary);
    list($t, $s)      = split(" - ",trim($ary[1]),2);
    $data['title']    = trim($t);
    $data['subtitle'] = trim($s);

    // Country and Year
    preg_match('/<TD.*?>Land und Jahr<\/TD>.*?<TD.*?>(.*) ([0-9]+)<\/TD>/i', $resp['data'], $ary);
    $data['year']     = trim($ary[2]);
    $contries = array();
    foreach (preg_split('/[\/,:]/',$ary[1]) as $country) 
    {
		$country = preg_replace('/[0-9]*/','',$country);
		$countries[] = trim($country);
    }
    $data['country']  = trim(join(', ',array_unique($countries)));

    // Cover-URL
    preg_match('/src="(\/coverimage\.php.*?)"/i', $resp['data'], $ary);
    $data['coverurl'] =  $dvdpalaceServer.trim($ary[1])."&pic.jpg";

    // Runtime
    preg_match('/>Laufzeit:.*?([0-9]+).*?min<\/TD>/i', $resp['data'], $ary);
    $data['runtime']  = trim($ary[1]);

    // Director
    preg_match('/<TD.*?>Regisseur\(e\)<\/TD>.*?<TD.*?>(.*?)<\/TD>/i', $resp['data'], $ary);
    preg_match('/<a.*?>(.*?)<\/a>/si', $ary[1], $ary);
    array_shift($ary);
    $data['director'] = trim(join(', ', $ary));

    // Rating
    preg_match_all('/<h5>Bewertung des Films.*?<br>.*?<IMG.*?>.*?([0-9][0-9]) von [0-9][0-9]/si', $resp['data'], $ary,PREG_PATTERN_ORDER);
    $count = 0;
    $sum = 0;
    while (isset($ary[1][$count])) 
    {
    	$sum += intval($ary[1][$count]);
        $count++;
    }
    if ($count != 0) 
    {
    	$data['rating'] = round($sum / $count); 
    	$data['comment'] .= "user rating: ".$data['rating']."\n";	
    } 

    // Languages
    // Languages (as Array)
	$laguages = array(
		'arabisch' => 'arabic',
		'bulgarisch' => 'bulgarian',
		'chinesisch' => 'chinese',
		'tschechisch' => 'czech',
		'dnisch' => 'danish',
		'holndisch' => 'dutch',
		'englisch' => 'english',
		'franzsisch' => 'french',
		'deutsch' => 'german',
		'griechisch' => 'greek',
		'ungarisch' => 'hungarian',
		'islndisch' => 'icelandic',
		'indisch' => 'indian',
		'israelisch' => 'israeli',
		'italienisch' => 'italian',
		'japanisch' => 'japanese',
		'koreanisch' => 'korean',
		'norwegisch' => 'norwegian',
		'polnisch' => 'polish',
		'portugisisch' => 'portuguese',
		'rumnisch' => 'romanian',
		'russisch' => 'russian',
		'serbisch' => 'serbian',
		'spanisch' => 'spanish',
		'schwedisch' => 'swedish',
		'thailndisch' => 'thai',
		'trkisch' => 'turkish',
		'vietnamesisch' => 'vietnamese',
		'kantonesisch' => 'cantonese',
		'katalanisch' => 'catalan',
		'zypriotisch' => 'cypriot',
		'zyprisch' => 'cypriot',
		'esperanto' => 'esperanto',
		'glisch' => 'gaelic',
		'hebrisch' => 'hebrew',
		'hindi' => 'hindi',
		'jdisch' => 'jewish',
		'lateinisch' => 'latin',
		'mandarin' => 'mandarin',
		'serbokroatisch' => 'serbo-croatian',
		'somalisch' => 'somali' 
	);
	$lang_list = array();
    preg_match('/<TD.*?>Tonformat\(e\)<\/TD>.*?<TD.*?>(.*?)<\/TD>/i', $resp['data'], $ary);
    preg_match_all('/(\w+):/si', $ary[1], $langs, PREG_PATTERN_ORDER);
    foreach($langs[1] as $language) {
		$language = trim(strtolower($language));
		$language = html_entity_decode(strip_tags($language));
		$language = preg_replace('/\s+$/','',$language);
		if (!$language) continue;
		if (isset($laguages[$language])) $language = $laguages[$language];
		else continue;
		if (!$language) continue;
		$lang_list[] = $language;
	}
    $data['language'] = trim(join(', ', array_unique($lang_list)));

    // Plot
    if(preg_match('/<TD VALIGN="TOP" COLSPAN=4 BGCOLOR="#FFFFFF" class="DETAILTEXT">(.*?)<\/TD>/i', $resp['data'], $ary))
    {
	    $ary[1] = preg_replace('/<div.*?>.*?<\/div>/','',$ary[1]);
	    $ary[1] = preg_replace('/<br.*?>/','',$ary[1]);
	    $data['plot'] = trim(html_entity_decode(strip_tags($ary[1])));
    }

    // Genres (as Array)
    $genres = array(
		'Thriller' => 'Thriller', 
		'Kriminalfilm' => 'Crime', 
		'Science Fiction' => 'Sci-Fi', 
		'Kinderfilm' => 'Family',
		'Dokumentation' => 'Documentary', 
		'Action' => 'Action', 
		'Drama' => 'Drama',
		'Abemnteuer' => 'Adventure',
		'Historienfilm' => '',
		'Komdie' => 'Comedy',
		'Romanze' => 'Romance',
		'Musik' => 'Musical',
		'Horror' => 'Horror',
		'Western' => 'Western',
		'Erotik' => '',
		'Eastern' => '',
		'Musikfilm' => 'Musical',
		'Trickfilm' => 'Animation',
		'Anime' => 'Animation',
		'Action / Thriller' => 'Thriller',
		'Animation' => 'Animation',
		'Drama / Dokumentation' => 'Drama',
		'Erotik / Thriller' => 'Thriller',
		'Fantasy' => 'Fantasy',
		'Filmoperette' => 'Musical',
		'Horrorkomdie' => 'Comedy',
		'Komdie / Thriller' => 'Comedy',
		'Kriegsfilm' => 'War',
		'Musikdokumentation' => 'Documentary',
		'Mystery' => 'Mystery',
		'Roadmovie' => 'Thriller',
		'Satire' => 'Comedy',
		'Science Fiction / Horror' => 'Horror',
		'Thriller / Drama' => 'Thriller',
		'Westernkomdie' => 'Western',
		'Zeichentrick' => 'Animation'
	);

	preg_match('/<TR CLASS="DARKBLUE">(.*?)<\/TR>/i',$resp[data],$ary);
	preg_match_all('/<TD.*?>(.*?)<\/TD>/si', $ary[1], $tds);
	
	// FSK
	$tds[1][1] = preg_replace('/&nbsp;/','',$tds[1][1]);
	$data['comment'] .= trim($tds[1][1])."\n";
	
	// Genre
	$tds[1][4] = preg_replace('/&nbsp;/','',$tds[1][4]);
	trim($tds[5]);
	$gens = preg_split("/ \/ /", trim($tds[1][4]));
	foreach($gens as $genre) {
		$genre = trim(html_entity_decode($genre));
		$genre = strip_tags($genre);
		if (!$genre) continue;
		if (isset($genres[$genre])) $genre = $genres[$genre];
		if (!$genre) continue;
		$data[genres][] = $genre;
	}

    // Cast
    preg_match('/<TD.*?>Darsteller \/ Sprecher<\/TD>.*?<TD.*?>(.*?)<\/TD>/i', $resp['data'], $ary);
    //preg_match('^\s*(.*)\s*$',html_entity_decode(strip_tags($ary[1])),$ary);
    $ary[1] = trim(html_entity_decode(strip_tags($ary[1])));
    $actors = preg_split("/, /", $ary[1]);
    $casts = "";
    $role = "";
    $actorid = "";
	foreach($actors as $actor) {
		$actor = preg_replace('/\s+$/','',$actor);
		if (preg_match('/:$/',$actor)) continue;
		// $casts .= $actor."::".$role."::".$actorid."\n";
		$casts .= $actor."\n";
		$actorid = "";
	}
	$data['cast'] = trim($casts);
	
	// EAN-Code
	preg_match('/<TD.*?>EAN-Code<\/TD>.*?<TD.*?><b>(.*?)<\/b><\/TD>/i', $resp['data'], $ary);
	$data['diskid'] = trim($ary[1]);
	
	// Aspect Ratio
	preg_match('/<TD.*?>Bildformat\(e\)<\/TD>.*?<TD.*?>(.*?)<\/TD>/i', $resp['data'], $ary);
	$data['comment'] .= trim(html_entity_decode(strip_tags($ary[1])));
	 
    return $data;
}

/**
 * Parses Actor-Details
 *
 * Fetch covers and URLs of movies with this actor
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param  string  $name  Name of the Actor
 * @return array          array with Movie-URL and Thumbnail
 */
function dvdpalaceMoviesByActor($name)
{
    global $dvdpalaceServer;
    global $resultsPerPage;
    
    // fetch data by name
	$pageNo = 1;
    while(1) {
	    $url = '/dvddatabase/dbsearch.php?action=2&suchbegriff='.urlencode($name).'&start='.(($pageNo - 1) * $resultsPerPage);
	    $resp = httpClient($dvdpalaceServer.$url, 1);
	    $resp['data'] = preg_replace('/[\r\n\t]/',' ', $resp['data']);
	
	    $single = array();
	    
	    // multiple matches
	    $matchcount = 0;
		if (preg_match('/<td class="tabcontentbgwhite".*?>(.*?)<\/td>/si', $resp['data'], $single)) 
	    {
			for ($matchcount=1;$matchcount <= count($single); $matchcount++)
			{
		    	switch ($matchcount % 3) {
		    		case 1: // Picture
		    			preg_match('/<img.*?src="(.*?)"/i', $single[1][$matchcount - 1], $purl);
		    			$ary[floor($matchcount/3)][1] = $dvdpalaceServer.$purl[1];
	   					break;	
		    		
		    		case 2: // Link & Name
		    			preg_match('/<a.*?href="(.*?)"/i', $single[1][$matchcount - 1], $lurl);
		    			$ary[floor($matchcount/3)][0] = $dvdpalaceServer.$lurl[1];
	   					break;	
		    		
		    		case 3: // Adds
						// not needed
	   					break;	
		    	}
			}
    	}	    	


		if (preg_match('#Seiten \(<b>([0-9]+)</b>\)#i', $resp['data'], $single)) {
			if ($pageNo == $single[1]) break; 
			else $pageNo++;  
		}
		else break;
    }
    
    return $ary;
}

?>
