<?php
/**
 * Search page
 *
 * Database searches for movies
 *
 * @package Search
 * @author  parts by Justin Pasher <justin@pasher.org>
 * @version $Id: search.php,v 2.25 2005/03/03 13:50:21 andig2 Exp $
 */

require_once './core/functions.php';
require_once './core/queryparser.php';
require_once './core/output.php';

// prepare fieldselect
$search_fields = array( 'title'     => $lang['title'],
                        'subtitle'  => $lang['subtitle'],
                        'director'  => $lang['director'],
                        'actors'    => $lang['cast'],
                        'plot'      => $lang['plot'],
                        'year'      => $lang['year'],
                        'country'   => $lang['country'],
                        'diskid'    => $lang['diskid'],
                        'owner'     => $lang['owner'],
                        'mediatype' => $lang['mediatype'],
                        'language'  => $lang['language'],
                        'comment'   => $lang['comment']);

// search fields to replace by physical name
$replace_fields = array('mediatype' => TBL_MEDIATYPES.'.name',
                        'owner'     => TBL_USERS.'.name',
                        'diskid'    => TBL_DATA.'.diskid');
            
// add custom fields
for ($i=1; $i<=4; $i++)
{
	if (!empty($config['custom'.$i])) 
    {
		$search_fields['custom'.$i] = $config['custom'.$i];
	}
}

// no fields given? Select all
if (empty($fields)) 
{
    // name search?
    if ($isname) {
        $fields = array('director', 'actors');
    } else {
        $fields = array_keys($search_fields);
    }
} 
// only one field? convert to array
elseif (!is_array($fields)) 
{
     $fields = array($fields);
}

// translate search fields into SQL column names, store original for template first
$smarty->assign('selected_fields', $fields);
foreach ($fields as $search_field)
{
    /*
     * field name conversion is required since the mysql AS columns can't be used in where clauses
     * add all fields here where the logical name in the template differs from the physical column name (e.g. mediatype)
     * or where the column name is ambiguous (e.g. diskid)
     */
    if (array_key_exists($search_field, $replace_fields))
    {
        if (($key = array_search($search_field, $fields)) !== false)
        {
            unset($fields[$key]);
            $fields[] = $replace_fields[$search_field];
        }        
    }        
}

// prepare search query
if (isset($q))
{
	$QUERY = '';

	if (!empty($q))
	{
		$error  = '';
		$tokens = queryparser($q, $error);

		$wild_char = (empty($nowild)) ? '%' : '';

		foreach ($tokens as $token)
		{
            // escape search token
			$token['token'] = addslashes($token['token']);

            // concatenate tokens with token operator
			$QUERY .= $token['ops'].' (';

            // concatenate all searchable fields with OR
			foreach ($fields as $field) 
            {
				$QUERY .= " ($field LIKE '$wild_char".$token['token']."$wild_char') OR ";
			}

            // concatenate custom fields with OR
			for ($custom = 1; $custom <= 4; $custom++)
			{
				if (!empty($config['custom'.$custom]) &! empty($fields['custom'.$custom])) {
					$QUERY .= " (custom$custom LIKE '$wild_char".$token['token']."$wild_char') OR ";
				}
			}
			$QUERY .= ' 1=2)';
		}
	}
	
    // filter by genres
	if (count($genres))
	{
        $TABLE  = TBL_VIDEOGENRE.',';
        $QUERY .= ' AND '.TBL_DATA.'.id = '.TBL_VIDEOGENRE.'.video_id AND (';

		foreach ($genres as $genre)
        {
            $FILTER .= 'OR '.TBL_VIDEOGENRE.'.genre_id = '.$genre.' ';
		}
		
        $FILTER = preg_replace('/^OR/', '', $FILTER);
		$QUERY .= $FILTER;
		$QUERY .= ')';
	}

    // XML / PDF export
    if ($export && $config[$export])
    {
        $func = $export.'export';
        require_once './core/'.$export.'.php';

        if (function_exists($func)) $func("1=1 $QUERY ORDER BY title, subtitle");
        exit();
    }

    $select = 'SELECT DISTINCT '.TBL_DATA.'.id, '.TBL_DATA.'.diskid, 
                    title, subtitle, language, 
                    year, director, LEFT(plot,300) AS plot, seen, 
                    imgurl, 
                    md5, comment, disklabel, imdbID, actors, runtime, 
                    country, filename, filesize, filedate, audio_codec, 
                    video_codec, video_width, video_height, istv, 
                    lastupdate, seen, mediatype, custom1, custom2, 
                    custom3, custom4, created,
                    '.TBL_LENT.'.who, '.TBL_USERS.'.name AS owner /*, mediatypes.name AS mediatype */
               FROM '.$TABLE.' '.TBL_DATA.' 
          LEFT JOIN '.TBL_LENT.' ON '.TBL_DATA.'.diskid = '.TBL_LENT.'.diskid 
          LEFT JOIN '.TBL_MEDIATYPES.' ON '.TBL_DATA.'.mediatype = '.TBL_MEDIATYPES.'.id
          LEFT JOIN '.TBL_USERS.' ON owner_id = '.TBL_USERS.".id
              WHERE 1=1
                    $QUERY
           ORDER BY title, subtitle";

    $result = runSQL($select);
}

// prepare templates
tpl_page('search', $q);
tpl_list($result);

$smarty->assign('q', $q);
$smarty->assign('q_q', formvar($q));
$smarty->assign('search_fields', $search_fields);
$smarty->assign('genreselect', out_genres($genres));
$smarty->assign('engine', $config['engine']);

// allow XML export
if (isset($q))
{
    $link = $_SERVER['QUERY_STRING'];
    if ($link) $link .= '&';
    $link = 'search.php?'.$link;
    
    if ($config['xml']) $smarty->assign('xml', $link);
    if ($config['pdf']) $smarty->assign('pdf', $link);
}


// person search?
if ($isname && ($config['actorpics']))
{
	$smarty->assign('imgurl', getActorThumbnail(urldecode(preg_replace('/&quot;|"/', '', formvar($q)))));
}

// display templates
smarty_display('header.tpl');
flush();

smarty_display('search.tpl');
if (isset($q)) smarty_display('list.tpl');
smarty_display('footer.tpl');

?>
